<?php
/**
 * FileName  Data.php.
 * @project: skshop-magento2
 * @developer: Larisa Groza
 * @company: BitStone
 */

namespace KPIs\Customer\Helper;

use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Customer\Model\CustomerFactory;
use KPIs\Customer\Logger\Logger;
use Magento\Customer\Model\GroupFactory;

class Data extends AbstractHelper
{
    /**
     * @var CustomerFactory
     */
    protected $customerFactory;

    /**
     * Logger of customer's log data.
     *
     * @var Logger
     */
    protected $logger;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfigObject;

    /**
     * @var GroupFactory
     */
    protected $group;

    /**
     * Data constructor.
     * @param CustomerFactory $customerFactory
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfigObject
     * @param GroupFactory $group
     * @param Logger $logger
     */
    public function __construct(
        CustomerFactory $customerFactory,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfigObject,
        GroupFactory $group,
        Logger $logger
    )
    {
        $this->customerFactory = $customerFactory;
        $this->logger = $logger;
        $this->scopeConfigObject = $scopeConfigObject;
        $this->group = $group;
    }

    public function getCustomerAccessPaid()
    {
        return [
            'sk Preview' => 1,
            'sk Insight' => 2,
            'sk Explorer' => 3,
            'sk Premium' => 4,
            'sk Standard' => 5,
            'TKI insight+' => 6,
            'TKI Membership' => 7,
        ];
    }

    public function getCustomerMembership()
    {
        return [
            'Paid' => 1,
            'Participant' => 2,
            'Team' => 3,
        ];
    }

    public function getAllCustomerAccess()
    {
        return [
            'sk Preview' => 1,
            'sk Insight' => 2,
            'sk Explorer' => 3,
            'sk Premium' => 4,
            'sk Standard' => 5,
            'TKI insight+' => 6,
            'TKI Membership' => 7,
            'sk Community' => 8,
            'sk Community-ex sk Preview' => 9,
            'sk Community-ex sk Explorer' => 10,
            'sk Community-ex sk Standard' => 11,
            'sk Community-ex sk Premium' => 12,
            'sk Community-ex TKI m' => 13,
            'sk Community-ex TKI insight+ m' => 14,
        ];
    }

    public function getCustomerAccessForProduct()
    {
        return [
            'sk Preview' => 1,
            'sk Insight' => 2,
            'sk Explorer' => 3,
            'sk Premium' => 4,
            'sk Standard' => 5,
            'TKI insight+' => 6,
            'TKI Membership' => 7,
        ];
    }

    /**
     * Method used in updating the customer attributes regarding the kpiSync.
     *
     * @param $customerId
     * @param $updateValues
     * @param boolean $send
     */
    public function updateKpiViewsAndAccess($customerId, $updateValues, $send = true)
    {
        $customer = $this->customerFactory->create()->load($customerId);
        if ($customer) {
            try {
                $customerData = $customer->getDataModel();
                $customerData->setId($customerId);
                foreach ($updateValues as $key => $value) {
                    $customerData->setCustomAttribute($key, $value);
                }
                $customer->updateData($customerData);
                $customer->save();
                if ($send && isset($updateValues['kpi_views_customer'])) {
                    $this->sendSmartkpisViews($customerId, $updateValues['kpi_views_customer']);
                }
            } catch (\Exception $e) {
                $this->logger->debug('Exception in updateKpiViewsAndAccess: ' . $e->getMessage());
            }
        }
    }

    /**
     * Send kpiViews to the MainWebsite.
     *
     * @param $customerId
     * @param $kpiViews
     * @return bool
     */
    public function sendSmartkpisViews($customerId, $kpiViews)
    {
        $endpointUrl = $this->scopeConfigObject->getValue('kpi_views_sync/general/endpoint_url');
        if ($endpointUrl) {
            $url = $endpointUrl . $customerId . '/' . $kpiViews;

            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $url);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
            $response = curl_exec($ch);
            try {
                if (!curl_errno($ch)) {
                    return true;
                }
            } catch (Exception $e) {
                $this->logger->debug('Exception in sendSmartkpisViews: ' . $e->getMessage());
            }
        }
        return false;
    }

    /**
     * Magento user sent over to SmartKpiInstitue.
     *
     * @param $customerSession
     * @return array
     */
    public function getCustomerForSmartkpis($customerSession)
    {
        if ($customerSession->isLoggedIn()) {
            $customer = $customerSession->getCustomer();
            $group = $this->getCustomerGroupById($customer->getGroupId());
            $customerAccess = ($customer->getCustomerAccess()) ? $this->getCustomerAccessByValue($customer->getCustomerAccess()) : '';
            $magentoUser = $customer->toArray();
            unset($magentoUser['password_hash']);
            $magentoUser['group'] = $group;
            $magentoUser['access'] = $customerAccess;
            return $magentoUser;
        }
        return [];
    }

    /**
     * Get customer group name.
     *
     * @param $groupId
     * @return string
     */
    private function getCustomerGroupById($groupId)
    {
        $groupModel = $this->group->create()->load($groupId);
        return $groupModel->getCustomerGroupCode();
    }

    /**
     * Get customer access label.
     *
     * @param $customerAccessId
     * @return mixed
     */
    private function getCustomerAccessByValue($customerAccessId)
    {
        $allCustomerAccess = array_flip($this->getAllCustomerAccess());
        return $allCustomerAccess[$customerAccessId];
    }

    /**
     * Api login url.
     * 
     * @return mixed
     */
    public function getSmartkpisLoginUrl()
    {
        return $this->scopeConfigObject->getValue('kpi_login_sync/general/login_url');
    }

    /**
     * Api logout url.
     *
     * @return mixed
     */
    public function getSmartkpisLogoutUrl()
    {
        return $this->scopeConfigObject->getValue('kpi_login_sync/general/logout_url');
    }
}