<?php
/**
 * ClassName  SetKpisViews.
 * @developer: Larisa Groza
 * @company: BitStone
 */

namespace KPIs\Customer\Observer;

use Magento\Catalog\Model\ProductFactory;
use Magento\Customer\Model\CustomerFactory;
use Magento\Framework\Stdlib\DateTime;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use KPIs\Customer\Logger\Logger;

class OrderChange implements ObserverInterface
{
    /**
     * @var CustomerFactory
     */
    protected $customerFactory;

    /**
     * @var \Magento\Catalog\Model\ProductFactory
     */
    protected $productFactory;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var \KPIs\Customer\Helper\Data
     */
    protected $helper;
    /**
     * Logger of customer's log data.
     *
     * @var Logger
     */
    protected $logger;


    /**
     * OrderStatusChange constructor.
     * @param CustomerFactory $customerFactory
     * @param ProductFactory $productFactory
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \KPIs\Customer\Helper\Data $helper
     * @param Logger $logger
     */
    public function __construct(
        CustomerFactory $customerFactory,
        \Magento\Catalog\Model\ProductFactory $productFactory,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \KPIs\Customer\Helper\Data $helper,
        Logger $logger
    ) {
        $this->customerFactory = $customerFactory;
        $this->productFactory = $productFactory;
        $this->storeManager = $storeManager;
        $this->helper = $helper;
        $this->logger = $logger;
    }

    /**
     * Handler for 'kpis_views_update_on_order_complete' event triggered when order status changes
     *
     * @param Observer $observer
     * @return void
     */
    public function execute(Observer $observer)
    {
        $order = $observer->getOrder();
        $orderState = $order->getState();
        $customerId = $order->getCustomerId();
        $flagKpisGranted = $order->getSkNotificationStatus();
        if ($orderState && $customerId && in_array($orderState, array(\Magento\Sales\Model\Order::STATE_COMPLETE, \Magento\Sales\Model\Order::STATE_PROCESSING)) && $flagKpisGranted != 1) {
            $kpisForOrder = $this->_getKpisFromOrder($order);
            if (sizeof($kpisForOrder)) {
                list($productCustomerAccess, $expirationDate, $kpiViews) = $kpisForOrder;

                $customerMembership = $this->helper->getCustomerMembership()['Paid'];
                $customerAccess = $this->helper->getCustomerAccessForProduct()[$productCustomerAccess];
                $updateCustomerAttributes = [
                    'kpi_views_customer' => $kpiViews,
                    'customer_membership' => $customerMembership,
                    'valid_until' => $expirationDate,
                    'customer_access' => $customerAccess
                ];

                $order->setSkNotificationStatus(1)
                    ->save();
            }
        }

        if ($orderState && $customerId && in_array($orderState, array(\Magento\Sales\Model\Order::STATE_CANCELED, \Magento\Sales\Model\Order::STATE_CLOSED)) && $flagKpisGranted == 1) {
            $kpisForOrder = $this->_getKpisFromOrder($order);
            if (sizeof($kpisForOrder)) {
                list($productCustomerAccess, $expirationDate, $kpiViews) = $kpisForOrder;
                $customerMembership = '';
                $customerAccess = 1;
                $updateCustomerAttributes = [
                    'kpi_views_customer' => $kpiViews,
                    'customer_membership' => $customerMembership,
                    'valid_until' => '',
                    'customer_access' => $customerAccess
                ];
            }
        }
        if (isset($updateCustomerAttributes)) {
            $this->helper->updateKpiViewsAndAccess($customerId, $updateCustomerAttributes);
        }
    }

    /**
     * Get list of customer access type, validity and number of kpis.
     *
     * @param \Magento\Sales\Model\Order $order
     * @return array
     */
    private function _getKpisFromOrder(\Magento\Sales\Model\Order $order)
    {
        $items = $order->getAllItems();
        $storeId = $this->storeManager->getStore()->getId();
        foreach ($items as $item) {
            $productId = $item->getProductId();
            $product = $this->productFactory->create()->load($productId);

            $kpiViews = $product->getResource()->getAttributeRawValue(
                $productId,
                'kpi_views_product',
                $storeId
            );

            if (!empty($kpiViews) && $kpiViews != 0) {
                $customerAccess = $product->getResource()->getAttribute('customer_access')->getFrontend()->getValue($product);
                $period = $product->getResource()->getAttribute('period')->getFrontend()->getValue($product);

                $expireDate = date('Y-m-d', strtotime("+ $period month"));
                return [$customerAccess, $expireDate, $kpiViews];
            }
        }
        return [];
    }
}